#include "ModelLoader.h"

/*----------------------------------------------------------------------------------*\
 *	file:	ModelLoader.c
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
*		Includes
\*----------------------------------------------------------------------------------*/

/* c std library includes */
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* windows compilation requires windows.h */
#ifdef WIN32
	/* This speeds up compile time on WIN32 by excluding lots of rubbish stuff */
	#define WIN32_LEAN_AND_MEAN	
	#include <Windows.h>
#endif

/* open gl includes */
#include <GL/gl.h>
#include <GL/GLU.h>

/* custom includes */

#include "ModelLoader.h"

/*----------------------------------------------------------------------------------*\
*		Data Types
\*----------------------------------------------------------------------------------*/
/*
*	Define a new type called 'vec' that represents a 3D vector
*/
struct vec 
{
	float x;
	float y;
	float z;
};

/*
*	At the moment our data type is called 'struct vec'. To make our 
*	lives easier we rename that to just vec by using the typedef.
*/
typedef struct vec vec;

/*
 *	Define a new type called 'uv' that represents a 2D texture co-ordinate
 */
struct uv 
{
	float u;
	float v;
};

/*
 *	Rename 'struct uv' to 'uv' with the use of typedef.
 */
typedef struct uv uv;

/*
*	Define a new type called 'tri' that represents the vector, normal and texture values
*	in respect to x, y and z
*/
struct tri
{
	int v[3];
	int n[3];
	int t[3];
};
/*
*	At the moment our data type is called 'tri'.
*	we rename that to just tri by using the typedef.
*/
typedef struct tri tri;

/*
 *	structure to hold data read file data.
 *  the mesh structure forms a node on a linked list,
 *   hence the pointer next.
 */
struct mesh
{
	/*
	 *	Vertex data
	 */
	int numVertices;
	vec* VertexArray;

	/*
	 *	Normal data
	 */
	int numNormals ;
	vec* NormalArray;

	/*
	 *	Texturing co-ordinate data
	 */
	int numuv;
	uv* uvarray;

	/*
 	 *	Face data
	 */
	int numFaces ;
	tri* Faces;

	/*
	 *	The id number of the mesh
	 */
	int id;

	/*
	 *	A pointer to the next node in the list
	 */
	struct mesh* next;
};
/*
*	At the moment our data type is called 'mesh'.
*	we rename that to just mesh by using the typedef.
*/
typedef struct mesh mesh;

/*----------------------------------------------------------------------------------*\
*		Globals
\*----------------------------------------------------------------------------------*/

/*
 *	first node points to nothing
 */
mesh* MeshHead = NULL;
/*
 *	this variable is used to generate unique id numbers for the mesh nodes
 */
int gid=0;
/*
 *	create the mesh 
 */
void CreateMesh() {
	/*
	 *	allocate some memory to what will end up being the size of the mesh
	 */
	mesh* p = (mesh*)malloc( sizeof(mesh) );
/*
 *	to ensure that everything is safe, we set all of the bytes within the 
 *	mesh structure to zero with the call :
 *	(this ensures all pointers are set to NULL and all data counts are zeroed)
 */
	memset(p,0,sizeof(mesh));
/*
 *	generate a unique id for each mesh created
 */
	p->id = ++gid;
/*
 *	p's point to the next node and MeshHead point to the same place
 */
	p->next=MeshHead;
/*
 *	MeshHead and p point to the same place
 */
	MeshHead = p;
}
/*
 *	get mesh id number
 */
mesh* getMesh (int id){
	/*
	 *	use pointer p to traverse linked list
	 */
	mesh* p = MeshHead;
	while (p != NULL) {
		/*
		 *	but if p is pointing to the mesh we want, return it
		 */
		if (p->id == id) {
			return p;
		}
		/*
		 *	p will then point to the next node there after
		 */
		p = p->next;
	}

	/* no mesh found */
	return NULL;
}

/*----------------------------------------------------------------------------------*\
*		Functions
\*----------------------------------------------------------------------------------*/
/*
 *	draw the mesh with id number 'id'
 */
void drawmod (int id)
{
	int i,j;

	/*
	 *	get pointer to mesh
	 */
	mesh* p = getMesh(id);
/*
 *	if p is valid
 */
	if(p!=NULL)
	{
		/*
		 *	start drawing the triangles and go through all 
		 *	the data for the uv, normal and texture values
		 */
		glBegin(GL_TRIANGLES);

		/* for each face */
		for(i=0; i<p->numFaces; i++)
		{
			/* for each vertex on face */
			for(j=0; j<3; j++){

				glTexCoord2fv ((float*)&p->uvarray[ p->Faces[i].t[j] ]);
				glNormal3fv ((float*)&p->NormalArray[ p->Faces[i].n[j] ]);
				glVertex3fv ((float*)&p->VertexArray[ p->Faces[i].v[j] ]);
			}
		}
		/*
		 *	stop drawing triangles
		 */
		glEnd();
	}

}
/*
 *	draw everything in list
 */
void drawall ()
{
	int i,j;

	/*
	*	use pointer p to traverse linked list
	*/
	mesh* p = MeshHead;
	while (p != NULL) 
	{
		/*
		 *	draw triangles
		 */
		glBegin(GL_TRIANGLES);
		for(i=0; i<p->numFaces; i++)
		{
			for(j=0; j<3; j++){

				glTexCoord2fv ((float*)&p->uvarray[ p->Faces[i].t[j] ]);
				glNormal3fv ((float*)&p->NormalArray[ p->Faces[i].n[j] ]);
				glVertex3fv ((float*)&p->VertexArray[ p->Faces[i].v[j] ]);
			}
		}
		/*
		 *	stop drawing the triangles
		 */
		glEnd();
		/*
		 *	move to next mesh
		 */
		p = p->next;
	}

}

/*
 *	deleting the model with specified id number
 */
void deletemod (int id)
{

	mesh* previous=NULL,*current;
	current = MeshHead;
/*
 *	traverse linked list
 */
	while (current != NULL)
	{
		/*
		 *	if mesh found
		 */
		if (current->id == id)
		{
			/*
			 *	and if previous is not NULL, then previous->next
			 *  needs to skip current...
			 */
			if (previous!=NULL)
				previous->next = current->next;
			else
				/* if removing first node in list */
				MeshHead = current->next;
			/*
			 *	free memory for current
			 */
			free (current);
			return;
		}
	/*
	 *	move to next node
	 */
	previous = current;
	current = current->next;
	
	}

}
/*
 *	\brief	This function loads the specified *.obj file.
 */
int loadmod (char* filename)
{

	/*
	 *	We use a file pointer to reference a file on the hard drive. This 
	 *	allows us to open more than one file at a time. In this case 'fp'
	 *	is our file pointer. 
	 *
	 *	We use 'fopen' to open a file. The first parameter is the filename,
	 *	the second is the mode; ie "r" = reading, "rb" = reading binary,
	 *	"w" = writing, "wb" = writing binary. 
	 */
	FILE* fp = fopen(filename, "r");
	
	/*
	 *	If the file pointer equals NULL, then the file could not be opened.
	 */
	if (fp == NULL) 
	{
		return 0;	
	}

	/*
	 *	Create a new mesh structure. This function always adds a mesh
	 *	to the start of the linked list. 
	 */
	CreateMesh();


	/*
	 *	While we are not at the end of the file
	 */
	while( !feof(fp) ) 
	{
		/*
		 *	This is an array of characters (ie, a text string). This will
		 *	be used to store a single piece of text that is read in from the file.
		 */
		char buffer[256];

		buffer[0]=NULL;

		/*
		 *	use 'fscanf' to read a single piece of text into buffer. 
		 */
		fscanf(fp,"%s", buffer);

		if ( strcmp(buffer,"v")==0 ) 
		{
			MeshHead->numVertices++;
		}
		if ( strcmp(buffer,"vn")==0 ) 
		{
			MeshHead->numNormals++;
		}
		if ( strcmp(buffer,"vt")==0 )
		{
			MeshHead->numuv++;
		}
		if ( strcmp(buffer, "f")==0 )
		{
			MeshHead->numFaces++;
		}
	}
	/*
	 *	Close the file
	 */
	fclose(fp);

	/*
	 *	At this point we will know how much data was contained within
	 *	the file. Print it out and see.
	 */
	printf ("VTX: %d\n", MeshHead->numVertices);
	printf ("NML: %d\n", MeshHead->numNormals);
	printf ("UVS: %d\n", MeshHead->numuv);
	printf ("FCS: %d\n", MeshHead->numFaces);

	/*
	 *	Allocate memory to hold the data from the file in arrays.
	 */
	MeshHead->VertexArray = (vec*)malloc(MeshHead->numVertices*sizeof(vec));
	MeshHead->NormalArray = (vec*)malloc(MeshHead->numNormals*sizeof(vec));
	MeshHead->uvarray     = (uv*)malloc(MeshHead->numuv*sizeof(uv));
	MeshHead->Faces		= (tri*)malloc(MeshHead->numFaces*sizeof(tri));

	/*
	 *	reset data counts
	 */
	MeshHead->numVertices = 0;
	MeshHead->numNormals = 0;
	MeshHead->numuv = 0;
	MeshHead->numFaces = 0;

	/*
	 *	Open the file again for a second time
	 */
	fp = fopen(filename, "r");
	while (!feof(fp)) 
	{
		/*
		 *	Read in a bit of text
		 */
		char buffer[256];
		buffer[0]=NULL;
		fscanf(fp,"%s", buffer);

		/*
		 *	If "v", read in vertex
		 */
		if ( strcmp(buffer,"v")==0 ) 
		{
			fscanf(fp,"%f%f%f", &MeshHead->VertexArray[MeshHead->numVertices].x,
								&MeshHead->VertexArray[MeshHead->numVertices].y,
								&MeshHead->VertexArray[MeshHead->numVertices].z);
			MeshHead->numVertices++;
		}

		/*
		 *	If "vn", read in normal
		 */
		if ( strcmp(buffer,"vn")==0 ) {
			fscanf(fp,"%f%f%f", &MeshHead->NormalArray[MeshHead->numNormals].x,
								&MeshHead->NormalArray[MeshHead->numNormals].y,
								&MeshHead->NormalArray[MeshHead->numNormals].z);
			MeshHead->numNormals++;
		}

		/*
		 *	If "vt", read in texturing coord
		 */
		if ( strcmp(buffer,"vt")==0 ) {
			fscanf(fp,"%f%f",	&MeshHead->uvarray[MeshHead->numuv].u,
								&MeshHead->uvarray[MeshHead->numuv].v);
			MeshHead->numuv++;
		}

		/*
		 *	If "f" read in the face indices. Note that the 
		 *	indices of the obj file start at 1, the indices
		 *	for the arrays start at zero, so remove 1 from all 
		 *	indices in the face.
		 */
		if ( strcmp(buffer,"f")==0) {
			fscanf(fp, "%d/%d/%d%d/%d/%d%d/%d/%d", 
				&MeshHead->Faces[MeshHead->numFaces].v[0],
				&MeshHead->Faces[MeshHead->numFaces].t[0],
				&MeshHead->Faces[MeshHead->numFaces].n[0],
				&MeshHead->Faces[MeshHead->numFaces].v[1],
				&MeshHead->Faces[MeshHead->numFaces].t[1],
				&MeshHead->Faces[MeshHead->numFaces].n[1],
				&MeshHead->Faces[MeshHead->numFaces].v[2],
				&MeshHead->Faces[MeshHead->numFaces].t[2],
				&MeshHead->Faces[MeshHead->numFaces].n[2]);

				--MeshHead->Faces[MeshHead->numFaces].v[0];
				--MeshHead->Faces[MeshHead->numFaces].t[0];
				--MeshHead->Faces[MeshHead->numFaces].n[0];
				--MeshHead->Faces[MeshHead->numFaces].v[1];
				--MeshHead->Faces[MeshHead->numFaces].t[1];
				--MeshHead->Faces[MeshHead->numFaces].n[1];
				--MeshHead->Faces[MeshHead->numFaces].v[2];
				--MeshHead->Faces[MeshHead->numFaces].t[2];
				--MeshHead->Faces[MeshHead->numFaces].n[2];

			MeshHead->numFaces++;
		}  
	}
	/*
	 *	close file
	 */
	fclose(fp);
/*
 *	return data
 */
	return MeshHead->id;	
}

