/// \brief	When accessing any node data within maya, you must use a function set
///			and attach it to the relevant MObject within maya. All function sets
///			are listed with the prefix MFn. The class MFn also contains the type
///			enumerations for the various maya nodes, in the case of a mesh, 
/// 
///									MFn::kMesh
/// 
///			

#define NT_APP


// includes
//
#include <maya/MFloatMatrix.h>
#include <maya/MFloatVector.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MMatrix.h>
#include <maya/MPlug.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>

#include <maya/MFnMesh.h>
#include <maya/MFnTransform.h>

#include <maya/MItDependencyNodes.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItMeshVertex.h>

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>

#include "mesh.h"


/// \brief	This function outputs the specified mesh to the specified file stream
/// 
void OutputMesh(std::ofstream& ofs,MObject &obj)
{
	// attach a function set to the mesh node. We access all of the meshes data 
	// through the function set
	// 
	MFnMesh fnMesh(obj);

	// write out some info about the mesh
	// 
	ofs << "MESH "
		<< fnMesh.name().asChar() 
		<< "\n\tVertexCount= "
		<< fnMesh.numVertices() 
		<< "\n\tNormalCount= "
		<< fnMesh.numNormals()
		<< "\n\tTexCoordCount= "
		<< fnMesh.numUVs()
		<< "\n\tFaceCount= "
		<< fnMesh.numPolygons()
		<< std::endl;

	// Maya has a tendancy of storing additional nodes and objects all over the place.
	// We may come accross meshes that are nodes used purely as history or other 
	// construction data. In our case, we want to ignore all 'intermediate objects'
	//
	if ( !fnMesh.isIntermediateObject() ) 
	{
		int i,j;

		// this will hold the returned vertex positions
		MPointArray vertices;
	
		// use the function set to get the points 
		fnMesh.getPoints(vertices);


		// the above function has a second optional parameter specifying the co-ordinate space
		// you want the points in, ie local or world. The problem is, I have never managed to
		// get it to return anything other than local space coords. One of the many 'joys'
		// of the API...
		//
		// To counter this problem, I've hacked together something that creates a transform
		// function set and binds it to the parent node. This should allow us to query the 
		// transform.
		//
		MFnTransform fnXform( fnMesh.parent(0) );

		// Here we just grab the transform matrix 
		//
		MMatrix mat = fnXform.transformationMatrix();


		// go through each vertex in the array and write it to the file 
		//
		ofs << "\n\tVertices\n";
		for(i=0;i!=vertices.length() ; ++i)
		{
			// transform the points from local space to world space
			//
			MPoint OutPoint = vertices[i] * mat;
					
			// output 
			//
			ofs << "\t\t" << OutPoint.x << " " << OutPoint.y << " " << OutPoint.z << "\n";
		}



		// this will store the normals
		MFloatVectorArray normals;

		// get the normals
		fnMesh.getNormals(normals);

		// just to be an utter arse, normals require you to have float matrices. 
		// Unfortunately someone decided not to allow you to convert from the 
		// MMatrix class to the MFloatMatrix class, therefore we have to do it
		// in a very very gay way...
		//
		MFloatMatrix fmat;

		for(i=0;i!=4;++i)
		{
			for(j=0;j!=4;++j)
			{
				fmat.matrix[i][j] = mat.matrix[i][j];
			}
		}

		// go through each normal in the array and write it to the file 
		//
		ofs << "\n\tNormals\n";
		for(i=0;i!=normals.length() ; ++i)
		{

			// transform the normal into world space
			//
			MFloatVector OutNormal = normals[i] * fmat;

			// we must normalise the vector to counter any scaling in the transformation
			//
			OutNormal.normalize();

			// output the normal 
			// 
			ofs << "\t\t" << OutNormal.x << " " << OutNormal.y << " " << OutNormal.z << "\n";
		}

		// these two arrays will hold the u & v texture co-ordinates
		//
		MFloatArray u_coords;
		MFloatArray v_coords;

		// get the uv coordinates
		//
		ofs << "\n\tTexCoords\n";
		fnMesh.getUVs(u_coords,v_coords);

		// go through each uv in the array and write it to the file 
		//
		for(i=0;i!=u_coords.length() ; ++i)
		{
			ofs << "\t\t" << u_coords[i] << " " << v_coords[i] << "\n";
		}


		// to get at the face data, we need to use a mesh polygon iterator.
		//
		MItMeshPolygon itPoly(fnMesh.object());

		// keep looping until the iterator has finished
		// 
		while ( !itPoly.isDone() )
		{

			// get the number of vertices in this face
			unsigned int vcount = itPoly.polygonVertexCount();

			// output some info about the face
			//
			ofs << "\n\tFACE "
				<< vcount 
				<< std::endl;

			// for each vertex on the face....
			//
			for(i = 0; i!= vcount; ++i)
			{
				// have to get the uv index seperately
				int uv_index;
				itPoly.getUVIndex(i,uv_index);

				// output indices for this vertex
				ofs << "\t\t" << itPoly.vertexIndex(i) << " " << itPoly.normalIndex(i) << " " << uv_index << "\n";
			}

			// move to next poly in the mesh
			//
			itPoly.next();
		}
	}
}
