//	
//	This example uses MLibrary::Initialise to start maya in our own console application. 
//	
//	The first command line arg is a name of a maya file. We can load this file and then
//	specifically read back information we want from the file and write it to our own file.
//	
//	To make life simple, this exporter merely traverses through all meshes in the scene 
//	and outputs them to the file. 
//
//  You may notice certain bits of maya API crapness along the way....
//	

#define NT_APP


// include each header for each class you need. Gets a bit laborious later on...
//
#include <maya/MFloatMatrix.h>
#include <maya/MMatrix.h>
#include <maya/MLibrary.h>
#include <maya/MFnDagNode.h>
#include <maya/MPointArray.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MFnMesh.h>
#include <maya/MFnTransform.h>
#include <maya/MFileIO.h>
#include <maya/MItDependencyNodes.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItMeshVertex.h>

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>

// link to some fairly funky maya libs. NOTE: The Image library only became part of maya
// from version 4.01 onwards. It is not available in version 4.0
//
#pragma comment(lib,"Foundation.lib")
#pragma comment(lib,"OpenMaya.lib")
#pragma comment(lib,"Image.lib")


int main(int argc,char** argv)
{


	// if the argument count is one, then no files were specified
	//
	if (argc==1) {
		std::cerr << "[ USAGE ] Exporter <infile.mb> { <outfile.ext> }" << std::endl;
		return 0;
	}



	// var to hold the output file name
	std::string outfilename;

	// if only one argument was specified, I'll assume that it is a maya binary
	// or ascii file, in which case use remove the 'mb' or 'ma' and add a 'mod' extension
	// to the end of the filename 
	// 
	if (argc == 2 )
	{
		outfilename = argv[1];
		outfilename.resize(outfilename.size()-2);	// strip extension

		outfilename.push_back('m');					// add new extension
		outfilename.push_back('o');
		outfilename.push_back('d');
	}
	else
	{
		// if outfile file was specified, use that
		//
		outfilename = argv[2];
	}



	// initialise the maya library - This basically starts up Maya
	// 
	MLibrary::initialize(argv[0]);



	// open the file that the user specified by using the static methods contained
	// within the MFileIO class. 
	//
	if( MS::kSuccess == MFileIO::open( argv[1],	// file to load
									   NULL,	// allows you to specify the file type, ie "ma", "mb" etc (i think!)
									   true) )	// if false will not allow unsaved files to be deleted, but we don't care so always force it to open
	{

		// create an output file stream
		//
		std::ofstream ofs(outfilename.c_str());

		// make sure it opened OK
		//
		if (!ofs) {
			std::cerr << "Could not open output file " << outfilename << std::endl;

			// cleanup the Maya library data before exiting
			MLibrary::cleanup();
			return 1;
		}




		// iterate through the mesh nodes in the scene
		//
		MItDependencyNodes itDep(MFn::kMesh);

		// we have to keep iterating until we get through all of the nodes in the scene
		//
		while (!itDep.isDone()) 
		{

			// attach a function set to the mesh node. We access all of the meshes data 
			// through the function set
			// 
			MFnMesh fnMesh(itDep.item());


			// write out some info about the mesh
			// 
			ofs << "MESH "
				<< fnMesh.name().asChar()	//	get name

				<< "\n\tVertexCount= "
				<< fnMesh.numVertices()		//	get vertex count
				
				<< "\n\tNormalCount= "
				<< fnMesh.numNormals()		//	get normal count

				<< "\n\tTexCoordCount= "
				<< fnMesh.numUVs()			//	get tex coord count 
				
				<< "\n\tFaceCount= "
				<< fnMesh.numPolygons()		//	get the number of polygons
				
				<< std::endl;


			// Maya has a tendancy of storing additional nodes and objects all over the place.
			// We may come accross meshes that are nodes used purely as history or other 
			// construction data. In our case, we want to ignore all 'intermediate objects'
			//
			if ( !fnMesh.isIntermediateObject() ) 
			{
				int i,j;

				// this will hold the returned vertex positions
				MPointArray vertices;

				// use the function set to get the points 
				fnMesh.getPoints(vertices);


				// the above function has a second optional parameter specifying the co-ordinate space
				// you want the points in, ie local or world. The problem is, I have never managed to
				// get it to return anything other than local space coords. One of the many 'joys'
				// of the API...
				//
				// To counter this problem, I've hacked together something that creates a transform
				// function set and binds it to the parent node. This should allow us to query the 
				// transform.
				//
				MFnTransform fnXform( fnMesh.parent(0) );

				// Here we just grab the transform matrix 
				//
				MMatrix mat = fnXform.transformationMatrix();


				// go through each vertex in the array and write it to the file 
				//
				ofs << "\n\tVertices\n";
				for(i=0;i!=vertices.length() ; ++i)
				{
					// transform the points from local space to world space
					//
					MPoint OutPoint = vertices[i] * mat;
					
					// output the transformed point
					//
					ofs << "\t\t" << OutPoint.x << " " << OutPoint.y << " " << OutPoint.z << "\n";
				}



				// this will store the normals
				MFloatVectorArray normals;

				// get the normals
				fnMesh.getNormals(normals);

				// just to be an utter arse, normals require you to have float matrices. 
				// Unfortunately someone decided not to allow you to convert from the 
				// MMatrix class to the MFloatMatrix class, therefore we have to do it
				// in a very very gay way...
				//
				MFloatMatrix fmat;

				for(i=0;i!=4;++i)
				{
					for(j=0;j!=4;++j)
					{
						fmat.matrix[i][j] = mat.matrix[i][j];
					}
				}

				// go through each normal in the array and write it to the file 
				//
				ofs << "\n\tNormals\n";
				for(i=0;i!=normals.length() ; ++i)
				{

					// transform the normal into world space
					//
					MFloatVector OutNormal = normals[i] * fmat;

					// we must normalise the vector to counter any scaling in the transformation
					//
					OutNormal.normalize();

					// output the normal 
					// 
					ofs << "\t\t" << OutNormal.x << " " << OutNormal.y << " " << OutNormal.z << "\n";
				}



				// these two arrays will hold the u & v texture co-ordinates
				//
				MFloatArray u_coords;
				MFloatArray v_coords;

				// get the uv coordinates
				//
				ofs << "\n\tTexCoords\n";
				fnMesh.getUVs(u_coords,v_coords);

				// go through each uv in the array and write it to the file 
				//
				for(i=0;i!=u_coords.length() ; ++i)
				{
					ofs << "\t\t" << u_coords[i] << " " << v_coords[i] << "\n";
				}


				// to get at the face data, we need to use a mesh polygon iterator.
				//
				MItMeshPolygon itPoly(fnMesh.object());


				// keep looping until the iterator has finished
				// 
				while ( !itPoly.isDone() )
				{

					// get the number of vertices in this face
					unsigned int vcount = itPoly.polygonVertexCount();

					// output some info about the face
					//
					ofs << "\n\tFACE "
						<< vcount 
						<< std::endl;

					// for each vertex on the face....
					//
					for(i = 0; i!= vcount; ++i)
					{
						// have to get the uv index seperately
						int uv_index;
						itPoly.getUVIndex(i,uv_index);

						// output indices for this vertex
						ofs << "\t\t" << itPoly.vertexIndex(i) << " " << itPoly.normalIndex(i) << " " << uv_index << "\n";
					}

					// move to next poly in the mesh
					//
					itPoly.next();
				}
			}

			// move to next mesh
			itDep.next();

			ofs << std::endl;
		}

		// close the output file
		ofs.close();
	}
	else
	{
		std::cerr << "Could not open file " << argv[1] << std::endl;
	}


	// cleanup maya library before exiting

	MLibrary::cleanup();

	return 0;
}