///-----------------------------------------------------------------------------
/// 
/// \file	Mesh.h
/// \author	Rob Bateman
/// \brief	a simple file loader for a simple file... This loader basically 
///			uses streaming operators to read and write the file. The data is 
///			stored in a set os basic classes, XVertex, XNormal, XTexCoord and
///			XFace.
/// 
///			The XMesh class then holds the data within arrays, these array types 
///			are typedefs of std::vector<XVertex> etc.
/// 
///-----------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////
#pragma once

#ifndef XMESH__H_
#define XMESH__H_

//////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/GL.h>

#include <vector>
#include <fstream>


//////////////////////////////////////////////////////////////////////////
/// A class to hold a vertex
/// 
class XVertex
{
	// declare streaming operators as friends
	friend std::istream &operator>>(std::istream&,XVertex&);

public:
	
	/// whoa! a constructor that doesn't initialise anything?
	/// Whats the point? It will only serve to initialise all
	/// values to 0 which I will write over anyway when reading in
	/// the file.
	/// 
	XVertex(){}

	/// dtor
	~XVertex(){}

	/// copy ctor
	XVertex(const XVertex& v) : x(v.x),y(v.y),z(v.z) {}

	/// a function to pass the data to opengl
	inline void gl() const {
		glVertex3f( x,y,z );
	}

	/// the vertex data
	float x;
	float y;
	float z;
};
//////////////////////////////////////////////////////////////////////////

// typedefs for readabilities sake
typedef std::vector<XVertex> XVertList;
typedef XVertList::iterator XVertIt;


//////////////////////////////////////////////////////////////////////////
/// a class to hold a normal
/// 
class XNormal
{
	friend std::istream &operator>>(std::istream&,XNormal&);
public:

	/// ctor
	XNormal(){}

	/// dtor
	~XNormal(){}

	/// copy ctor
	XNormal(const XNormal& v) : x(v.x),y(v.y),z(v.z) {}
			
	inline void gl() const {
		glNormal3f( x,y,z );
	}

	/// the normal data
	float x;
	float y;
	float z;
};
//////////////////////////////////////////////////////////////////////////

// typedefs for readabilities sake
typedef std::vector<XNormal> XNormList;
typedef XNormList::iterator XNormIt;

//////////////////////////////////////////////////////////////////////////
/// a class to hold a uv texture coord
/// 
class XTexCoord
{
	friend std::istream &operator>>(std::istream&,XTexCoord&);

public:

	/// ctor
	XTexCoord(){}

	/// dtor
	~XTexCoord(){}

	/// copy ctor
	XTexCoord(const XTexCoord& v_) : u(v_.u),v(v_.v) {}

	inline void gl() const {
		glTexCoord2f( u,v );
	}

	/// the tex coord data
	float u;
	float v;
};

//////////////////////////////////////////////////////////////////////////

// typedefs for readabilities sake
typedef std::vector<XTexCoord> XTexList;
typedef XTexList::iterator XTexIt;

/// this internal structure is used to package all indices
/// into 12bytes. This keeps the data access sequentual through
/// memory in the draw function
///
struct XPointIndex 
{
	/// streaming operator
	friend std::istream &operator>>(std::istream& ifs,XPointIndex& pi) {
		return ifs >> pi.v >> pi.n >> pi.t;
	}

	/// i prefer not to initialise array indices because it often 
	/// highlights problems with the code that would be otherwise
	/// undetected. ie, the below indices are probably going to crash
	/// the program if they are not initialised. If I set these to 
	/// zero then I would never be aware of this problem....
	/// 
	XPointIndex() {}

	/// the texture coordinate index
	unsigned int t;

	/// the normal index
	unsigned int n;

	/// the vertex index
	unsigned int v;
};


//////////////////////////////////////////////////////////////////////////
/// a class to hold a face
///
class XFace
{
	/// stream operator
	friend std::istream &operator>>(std::istream&,XFace&);

	/// the mesh class is a friend...
	friend class XMesh;


public:


	/// ctor - sets the pointer to NULL for that is good...
	XFace() : m_Points(NULL) {}

	/// dtor
	~XFace() {
		delete m_Points;
	}

	/// this function sets the number of indices in the face
	/// 
	void SetSize(unsigned int);

protected:

	/// the number of points in the polygon
	unsigned int m_NumPoints;

	/// an array of point indices
	XPointIndex *m_Points;
};


//////////////////////////////////////////////////////////////////////////

// typedefs for readabilities sake
typedef std::vector<XFace> XFaceList;
typedef XFaceList::iterator XFaceIt;



//////////////////////////////////////////////////////////////////////////

class XMesh
{
	friend std::istream &operator>>(std::istream&,XMesh&);

public:

	/// ctor
	XMesh() {};

	/// dtor
	~XMesh() {};

	/// this function draws the mesh data
	void gl();

protected:

	/// an array of faces
	XFaceList m_Faces;

	/// an array of texture coords
	XTexList  m_TexCoords;
	
	/// an array of normals
	XNormList m_Normals;

	/// an array of vertices
	XVertList m_Vertices;

	/// the mesh name 
	std::string m_Name;

};


#endif