/////////////////////////////////////////////////////////////////////////////////////////
/// 
///	\file	Light.cpp
/// \author	Rob Bateman			( robthebloke@hotmail.com )
/// \brief	This file defines the single function OutputLight which will write the light
///			data to the file. 
///
/////////////////////////////////////////////////////////////////////////////////////////



#ifndef NT_APP
#define NT_APP
#endif

/////////////////////////////////////////////////////////////////////////////////////////

#include "light.h"

#include <maya/MMatrix.h>

#include <maya/MFnAmbientLight.h>
#include <maya/MFnAreaLight.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnDirectionalLight.h>
#include <maya/MFnLight.h>
#include <maya/MFnPointLight.h>
#include <maya/MFnSpotLight.h>
#include <maya/MFnTransform.h>

#include <iostream>

/////////////////////////////////////////////////////////////////////////////////////////
/// \brief	This function outputs light data to the specifed filestream. The MObject
///			passed as an argument is the handle to the maya node.
/// \param	ofs	-	the output stream
///	\param	obj	-	the maya object
/// 
void OutputLight(std::ofstream& ofs,MObject &obj)
{
	// make sure we only get valid nodes here
	assert( !obj.isNull() );

	// make sure the MObject represents a light!
	assert( obj.hasFn(MFn::kLight) );

	// attach a light function set to the object
	MFnLight fnLight(obj);

	// if the object is an intermediate object, we will ignore it
	if ( !fnLight.isIntermediateObject() ) 
	{
		// get light color
		MColor color(0.0f,0.0f,0.0f);
		color = fnLight.color();

		// point at zero
		MVector p(0,0,0);

		// transform it by the light x-form matrix
		//
		MFnTransform fnXform( fnLight.parent(0) );

		// get position of the light
		//
		p = fnXform.transformation().translation(MSpace::kWorld);

		// output some light info
		ofs << "LIGHT "

			// output name
			<< fnLight.name().asChar()
			
			// output light type
			<< "\n\tType=\t" 
			<< obj.apiTypeStr()

			// output color 
			<< "\n\tColor=\t"
			<< color.r
			<< " "
			<< color.g
			<< " "
			<< color.b 

			// output intensity
			<< "\n\tIntensity=\t" 
			<< fnLight.intensity()

			// output light position
			<< "\n\tPosition=\t"
			<< p.x << " " << p.y << " " << p.z
			<< std::endl;


		// if you need to get specific attributes associated with specific
		// light types then use something like this....
		//
		switch(obj.apiType()) 
		{
		case MFn::kPointLight:
			{
				MFnPointLight fnPoint(obj);

			}
			break;
		case MFn::kAmbientLight:
			{
				MFnAmbientLight fnAmbient(obj);

			}
			break;
		case MFn::kSpotLight:
			{
				MFnSpotLight fnSpot(obj);

			}
			break;
		case MFn::kDirectionalLight:
			{
				MFnDirectionalLight fnDir(obj);

			}
			break;
		case MFn::kAreaLight:
			{
				MFnAreaLight fnArea(obj);

			}
			break;
		default:
			break;
		}
	}
}
