///-----------------------------------------------------------------------------
/// 
/// \file	Texture.h
/// \author	Rob Bateman
/// \brief	This file defines the XImageHeader and the XTexture classes. These 
///			are used to read a texture file from a texture file
/// 
///-----------------------------------------------------------------------------

#pragma once

#include <stdio.h>
#include <math.h>

/// \brief	This struct is used as a 2byte identifier just before an image
/// \note	sizeof(ImageHeader) == 2
/// 
struct XImageHeader
{
	/// the width of the image as the power of two
	unsigned char m_WidthPower:4;
	/// the height of the image as a power of two
	unsigned char m_HeightPower:4;

	/// the bits per pixel
	/// 0 = 8bit
	/// 1 = 16bit
	/// 2 = 24bit
	/// 3 = 32bit
	/// 
	unsigned char m_Bpp:2;

	/// 6bits for a rainy day...
	unsigned char m_Unused:6;
};

/// \brief	a class to hold a texture
/// 
class XTexture
{
public:
	
	/// ctor
	XTexture(void);

	/// dtor
	~XTexture(void);

	/// returns the width of the texture
	inline unsigned short GetWidth() const {
		return static_cast<unsigned short>(pow(static_cast<unsigned short>(2),m_Header.m_WidthPower));
	}

	/// returns the height of the texture
	inline unsigned short GetHeight() const {
		return static_cast<unsigned short>(pow(static_cast<unsigned short>(2),m_Header.m_HeightPower));
	}

	/// returns the bits per pixel of the texture
	inline unsigned short GetBitsPerPixel() const {
		return GetBytesPerPixel() << 3;
	}

	/// returns the bytes per pixel of the texture
	inline unsigned short GetBytesPerPixel() const {
		return 1 + m_Header.m_Bpp;
	}

	/// returns the bytes per pixel of the texture
	inline const unsigned char* const GetPixels() const {
		return m_Pixels;
	}

	/// this function reads a texture from the specified file pointer
	bool Read(FILE* fp);

protected:

	/// the image header
	XImageHeader   m_Header;

	/// a pointer to the pixel data
	unsigned char* m_Pixels;
};
