#pragma once

#include <vector>
#include <iostream>
#include <math.h>

#include "Object.h"

#ifndef OUT
#define OUT
#endif

class XVector
{
public:


	/// ctor
	XVector(float x_=0,
				float y_=0,
				float z_=0)
		: x(x_),y(y_),z(z_) {}

	/// copy ctor
	XVector(const XVector& q) : x(q.x),y(q.y),z(q.z) {}

	float x;
	float y;
	float z;
};

inline XVector LERP(const XVector& v1, const XVector &v2,const float &t)
{
	float invt = 1.0f-t;
	return XVector( ((v1.x*invt)+(v2.x*t)),
					((v1.y*invt)+(v2.y*t)), 
					((v1.z*invt)+(v2.z*t)) );
}


class XQuaternion
{
public:

	/// the streaming operator for the quaternion
	friend std::istream& operator >> (std::istream& ifs,XQuaternion &q) {
		return ifs >> q.x >> q.y >> q.z >> q.w;
	}

	/// ctor
	inline XQuaternion(float x_=0,
				float y_=0,
				float z_=0,
				float w_=1)
		: x(x_),y(y_),z(z_),w(w_) {}

	/// copy ctor
	inline XQuaternion(const XQuaternion& q) : x(q.x),y(q.y),z(q.z),w(q.w) {}

	/// this function turns the quaternion into a 4x4 rotation matrix
	/// 
	void AsMatrix(OUT float* matrix);

	/// this function spherically interpolates between two quaternions
	/// 
	friend XQuaternion SLERP(const XQuaternion &q1,const XQuaternion &q2,const float &t);

	/// returns the magnitude of the quaternion
	inline float Mag() const {
		return static_cast<float>( sqrt(w*w + x*x + y*y + z*z) );
	}

	/// returns the inverse of the quaternion (aka conjugate)
	inline XQuaternion operator-() const {
		return XQuaternion( -x,-y,-z,w );
	}

	/// normalises the quaternion
	inline const XQuaternion& Normalise() {
		float l_inv = 1.0f/Mag();
		x*=l_inv;
		y*=l_inv;
		z*=l_inv;
		w*=l_inv;
		return (*this);
	}

	/// returns a normalised version of this quaternion
	inline XQuaternion Normalised() const {
		float l_inv = 1.0f/Mag();
		return XQuaternion (x*l_inv,y*l_inv,z*l_inv,w*l_inv);
	}

	/// the quaternion data
	float x;
	float y;
	float z;
	float w;
};






class XBone
{

	friend class XBoneAnimation;
	friend std::istream& operator >> (std::istream& ifs,XBone &q);

public:
	typedef std::vector<unsigned int> ChildArray;
	typedef ChildArray::iterator ChildIt;

	XBone(void);

	~XBone(void);

	unsigned int GetID() const {
		return m_ID;
	}
	unsigned int GetParentID() const {
		return m_ParentID;
	}

	void DrawBones( std::vector< XBone* >&,
		            std::vector< XObject* >&,
					std::vector< XMaterial*>& ,XQuaternion = XQuaternion(1,1,1,1)); 

//protected:

	/// the ID number of this bone
	unsigned int m_ID;

	/// the ID number of the parent bone
	unsigned int m_ParentID;

	/// the reference numbers to the child joints
	ChildArray m_ChildJoints;

	/// the reference numbers to any parented geometry/lights etc
	ChildArray m_ChildObjects;

	//////////////////////////////////////////////////////////////////////////
	/// 
	/// 

	/// the scale of the transform. This will hopefully not be used!
	/// scale within animating hierarchies is not fun to deal with. 
	/// The one exception is when it is used on influence objects at the
	/// end of a hierarchy. This is infact the only reason I am using it
	XVector		m_Scale;

	/// the translation of the joint
	XVector		m_Translation;

	/// the joints rotation as a quaternion. 
	XQuaternion m_RotationOrient;

	/// the joints rotation as a quaternion. 
	XQuaternion m_Rotation;

	/// the joints rotation as a quaternion. 
	XQuaternion m_JointOrient;

	std::string name;

	//////////////////////////////////////////////////////////////////////////
};


















