/*----------------------------------------------------------------------------------*\
 *	file:	ModelLoader.c
 *	author:	
 *	date:	
 *	notes:	
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
*		Includes
\*----------------------------------------------------------------------------------*/

/* c std library includes */
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* windows compilation requires windows.h */
#ifdef WIN32
	/* This speeds up compile time on WIN32 by excluding lots of rubbish stuff */
	#define WIN32_LEAN_AND_MEAN	
	#include <Windows.h>
#endif

/* open gl includes */
#include <GL/gl.h>
#include <GL/GLU.h>

/* custom includes */
#include "Vector.h"
#include "ModelLoader.h"



/*----------------------------------------------------------------------------------*\
*		Defines
\*----------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------------*\
*		Data Types
\*----------------------------------------------------------------------------------*/

/*
 *	Define a new type called 'uv' that represents a 2D texture co-ordinate
 */
struct uv 
{
	float u;
	float v;
};

/*
 *	Rename 'struct uv' to 'uv' with the use of typedef.
 */
typedef struct uv uv;

struct tri
{
	int v[3];
	int n[3];
	int t[3];
};

typedef struct tri tri;

/*----------------------------------------------------------------------------------*\
*		Globals
\*----------------------------------------------------------------------------------*/

/*
 *	Vertex data
 */
int numVertices = 0;
vec* VertexArray = NULL;

/*
 *	Normal data
 */
int numNormals = 0;
vec* NormalArray = NULL;

/*
 *	Texturing co-ordinate data
 */
int numuv = 0;
uv* uvarray = NULL;

/*
 *	Face data
 */
int numFaces = 0;
tri* Faces = NULL;

/*----------------------------------------------------------------------------------*\
*		Functions
\*----------------------------------------------------------------------------------*/
void drawmod ()
{
	int i,j;
	glBegin(GL_TRIANGLES);
	for(i=0; i<numFaces; i++)
	{
		for(j=0; j<3; j++){

			glTexCoord2fv ((float*)&uvarray[ Faces[i].t[j] ]);
			glNormal3fv (NormalArray[ Faces[i].n[j] ].data);
			glVertex3fv (VertexArray[ Faces[i].v[j] ].data);
		}
	}
	glEnd();
}

/*
 *	\brief	This function loads the specified *.obj file.
 */
unsigned int loadmod (char* filename)
{
	unsigned int num;

	/*
	 *	We use a file pointer to reference a file on the hard drive. This 
	 *	allows us to open more than one file at a time. In this case 'fp'
	 *	is our file pointer. 
	 *
	 *	We use 'fopen' to open a file. The first parameter is the filename,
	 *	the second is the mode; ie "r" = reading, "rb" = reading binary,
	 *	"w" = writing, "wb" = writing binary. 
	 */
	FILE* fp = fopen(filename, "r");
	
	/*
	 *	If the file pointer equals NULL, then the file could not be opened.
	 */
	if (fp == NULL) 
	{
		return 0;	
	}

	/*
	 *	reset data counts
	 */
	numVertices = 0;
	numNormals = 0;
	numuv = 0;
	numFaces = 0;

	/*
	 *	The function 'feof' is used to check for the end of a file. It 
	 *	returns true if we are at the end of the file and zero otherwise.
	 *	
	 *	We use this function call in the while statement, to say 
	 *	"While we are not at the end of the file.... "
	 */
	while( !feof(fp) ) 
	{
		/*
		 *	This is an array of characters (ie, a text string). This will
		 *	be used to store a single piece of text that is read in from the file.
		 */
		char buffer[256];

		buffer[0]=NULL;

		/*
		 *	use 'fscanf' to read a single piece of text into buffer. This works in 
		 *	the same way as fscanf except that the first parameter is the file 
		 *	pointer.
		 */
		fscanf(fp,"%s", buffer);

		if ( strcmp(buffer,"v")==0 ) 
		{
			numVertices++;
		}
		if ( strcmp(buffer,"vn")==0 ) 
		{
			numNormals++;
		}
		if ( strcmp(buffer,"vt")==0 )
		{
			numuv++;
		}
		if ( strcmp(buffer, "f")==0 )
		{
			numFaces++;
		}
	}
	/*
	 *	Close the file
	 */
	fclose(fp);

	/*
	 *	At this point we will know how much data was contained within
	 *	the file. Print it out and see.
	 */
	printf ("VTX: %d\n", numVertices);
	printf ("NML: %d\n", numNormals);
	printf ("UVS: %d\n", numuv);

	/*
	 *	Allocate memory to hold the data from the file in arrays.
	 */
	VertexArray = (vec*)malloc(numVertices*sizeof(vec));
	NormalArray = (vec*)malloc(numNormals*sizeof(vec));
	uvarray     = (uv*)malloc(numuv*sizeof(uv));
	Faces		= (tri*)malloc(numFaces*sizeof(tri));

	/*
	 *	reset data counts
	 */
	numVertices = 0;
	numNormals = 0;
	numuv = 0;
	numFaces = 0;

	/*
	 *	Open the file again for a second time
	 */
	fp = fopen(filename, "r");
	while (!feof(fp)) 
	{
		/*
		 *	Read in a bit of text
		 */
		char buffer[256];
		buffer[0]=NULL;
		fscanf(fp,"%s", buffer);

		/*
		 *	If "v", read in vertex
		 */
		if ( strcmp(buffer,"v")==0 ) 
		{
			fscanf(fp,"%f%f%f", &VertexArray[numVertices].x,
								&VertexArray[numVertices].y,
								&VertexArray[numVertices].z);
			numVertices++;
		}

		/*
		 *	If "vn" read in normal
		 */
		if ( strcmp(buffer,"vn")==0 ) {
			fscanf(fp,"%f%f%f", &NormalArray[numNormals].x,
								&NormalArray[numNormals].y,
								&NormalArray[numNormals].z);
			numNormals++;
		}

		/*
		 *	If "vt" read in texturing coord
		 */
		if ( strcmp(buffer,"vt")==0 ) {
			fscanf(fp,"%f%f",	&uvarray[numuv].u,
								&uvarray[numuv].v);
			numuv++;
		}

		if ( strcmp(buffer,"f")==0) {
			fscanf(fp, "%d/%d/%d%d/%d/%d%d/%d/%d", 
				&Faces[numFaces].v[0],&Faces[numFaces].t[0],&Faces[numFaces].n[0],
				&Faces[numFaces].v[1],&Faces[numFaces].t[1],&Faces[numFaces].n[1],
				&Faces[numFaces].v[2],&Faces[numFaces].t[2],&Faces[numFaces].n[2]);

				--Faces[numFaces].v[0];
				--Faces[numFaces].t[0];
				--Faces[numFaces].n[0];
				--Faces[numFaces].v[1];
				--Faces[numFaces].t[1];
				--Faces[numFaces].n[1];
				--Faces[numFaces].v[2];
				--Faces[numFaces].t[2];
				--Faces[numFaces].n[2];

			numFaces++;
		}  
	}
	fclose(fp);

	num = glGenLists(1);
	glNewList(num, GL_COMPILE);

	drawmod();
	glEndList();

	free(VertexArray);
	free(NormalArray);
	free(uvarray);
	free(Faces);



	return num;	
}













