/*----------------------------------------------------------------------------------*\
 *	file:	ModelLoader.c
 *	author:	
 *	date:	
 *	notes:	
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
*		Includes
\*----------------------------------------------------------------------------------*/

/* c std library includes */
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* windows compilation requires windows.h */
#ifdef WIN32
	/* This speeds up compile time on WIN32 by excluding lots of rubbish stuff */
	#define WIN32_LEAN_AND_MEAN	
	#include <Windows.h>
#endif

/* open gl includes */
#include <GL/gl.h>
#include <GL/GLU.h>

/* custom includes */
#include "Vector.h"
#include "ModelLoader.h"



/*----------------------------------------------------------------------------------*\
*		Defines
\*----------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------------*\
*		Data Types
\*----------------------------------------------------------------------------------*/

/*
 *	Define a new type called 'uv' that represents a 2D texture co-ordinate
 */
struct uv 
{
	float u;
	float v;
};

/*
 *	Rename 'struct uv' to 'uv' with the use of typedef.
 */
typedef struct uv uv;


struct tri
{
	int v[3];
	int n[3];
	int t[3];
};

typedef struct tri tri;

struct mesh
{
	/*
	 *	Vertex data
	 */
	int numVertices;
	vec* VertexArray;

	/*
	 *	Normal data
	 */
	int numNormals ;
	vec* NormalArray;

	/*
	 *	Texturing co-ordinate data
	 */
	int numuv;
	uv* uvarray;

	/*
 	 *	Face data
	 */
	int numFaces ;
	tri* Faces;

	/*
	 *	The id number of the mesh
	 */
	int id;

	/*
	 *	A pointer to the next node in the list
	 */
	struct mesh* next;
};
typedef struct mesh mesh;

/*----------------------------------------------------------------------------------*\
*		Globals
\*----------------------------------------------------------------------------------*/


mesh* pMesh = NULL;

int gid=0;

void CreateMesh() {
	mesh* p = (mesh*)malloc( sizeof(mesh) );

	memset(p,0,sizeof(mesh));

	p->id = ++gid;

	p->next=pMesh;

	pMesh = p;
}

mesh* getMesh (int id){
	mesh* p = pMesh;
	while (p != NULL) {
		if (p->id == id) {
			return p;
		}
		p = p->next;
	}
	return NULL;
}

/*----------------------------------------------------------------------------------*\
*		Functions
\*----------------------------------------------------------------------------------*/
void drawmod (int id)
{
	int i,j;
	mesh* p = getMesh(id);

	if(p!=NULL)
	{
		glBegin(GL_TRIANGLES);
		for(i=0; i<p->numFaces; i++)
		{
			for(j=0; j<3; j++){

				glTexCoord2fv ((float*)&p->uvarray[ p->Faces[i].t[j] ]);
				glNormal3fv (p->NormalArray[ p->Faces[i].n[j] ].data);
				glVertex3fv (p->VertexArray[ p->Faces[i].v[j] ].data);
			}
		}
		glEnd();
	}

}
void drawall ()
{
	int i,j;
	mesh* p = pMesh;

	while (p != NULL) 
	{
		glBegin(GL_TRIANGLES);
		for(i=0; i<p->numFaces; i++)
		{
			for(j=0; j<3; j++){

				glTexCoord2fv ((float*)&p->uvarray[ p->Faces[i].t[j] ]);
				glNormal3fv (p->NormalArray[ p->Faces[i].n[j] ].data);
				glVertex3fv (p->VertexArray[ p->Faces[i].v[j] ].data);
			}
		}
		glEnd();
		p = p->next;
	}

}


void deletemod (int id)
{
	mesh* previous=NULL,*current;
	current = pMesh;

	while (current != NULL)
	{
		if (current->id == id)
		{
			if (previous!=NULL)
				previous->next = current->next;
			else
				pMesh = current->next;
			free (current);
			return;
		}
	
	previous = current;
	current = current->next;
	
	}

}
/*
 *	\brief	This function loads the specified *.obj file.
 */
int loadmod (char* filename)
{

	/*
	 *	We use a file pointer to reference a file on the hard drive. This 
	 *	allows us to open more than one file at a time. In this case 'fp'
	 *	is our file pointer. 
	 *
	 *	We use 'fopen' to open a file. The first parameter is the filename,
	 *	the second is the mode; ie "r" = reading, "rb" = reading binary,
	 *	"w" = writing, "wb" = writing binary. 
	 */
	FILE* fp = fopen(filename, "r");
	
	/*
	 *	If the file pointer equals NULL, then the file could not be opened.
	 */
	if (fp == NULL) 
	{
		return 0;	
	}

	/*
	 *	Create a new mesh structure. This function always adds a mesh
	 *	to the start of the linked list. 
	 */
	CreateMesh();


	/*
	 *	The function 'feof' is used to check for the end of a file. It 
	 *	returns true if we are at the end of the file and zero otherwise.
	 *	
	 *	We use this function call in the while statement, to say 
	 *	"While we are not at the end of the file.... "
	 */
	while( !feof(fp) ) 
	{
		/*
		 *	This is an array of characters (ie, a text string). This will
		 *	be used to store a single piece of text that is read in from the file.
		 */
		char buffer[256];

		buffer[0]=NULL;

		/*
		 *	use 'fscanf' to read a single piece of text into buffer. This works in 
		 *	the same way as fscanf except that the first parameter is the file 
		 *	pointer.
		 */
		fscanf(fp,"%s", buffer);

		if ( strcmp(buffer,"v")==0 ) 
		{
			pMesh->numVertices++;
		}
		if ( strcmp(buffer,"vn")==0 ) 
		{
			pMesh->numNormals++;
		}
		if ( strcmp(buffer,"vt")==0 )
		{
			pMesh->numuv++;
		}
		if ( strcmp(buffer, "f")==0 )
		{
			pMesh->numFaces++;
		}
	}
	/*
	 *	Close the file
	 */
	fclose(fp);

	/*
	 *	At this point we will know how much data was contained within
	 *	the file. Print it out and see.
	 */
	printf ("VTX: %d\n", pMesh->numVertices);
	printf ("NML: %d\n", pMesh->numNormals);
	printf ("UVS: %d\n", pMesh->numuv);
	printf ("FCS: %d\n", pMesh->numFaces);

	/*
	 *	Allocate memory to hold the data from the file in arrays.
	 */
	pMesh->VertexArray = (vec*)malloc(pMesh->numVertices*sizeof(vec));
	pMesh->NormalArray = (vec*)malloc(pMesh->numNormals*sizeof(vec));
	pMesh->uvarray     = (uv*)malloc(pMesh->numuv*sizeof(uv));
	pMesh->Faces		= (tri*)malloc(pMesh->numFaces*sizeof(tri));

	/*
	 *	reset data counts
	 */
	pMesh->numVertices = 0;
	pMesh->numNormals = 0;
	pMesh->numuv = 0;
	pMesh->numFaces = 0;

	/*
	 *	Open the file again for a second time
	 */
	fp = fopen(filename, "r");
	while (!feof(fp)) 
	{
		/*
		 *	Read in a bit of text
		 */
		char buffer[256];
		buffer[0]=NULL;
		fscanf(fp,"%s", buffer);

		/*
		 *	If "v", read in vertex
		 */
		if ( strcmp(buffer,"v")==0 ) 
		{
			fscanf(fp,"%f%f%f", &pMesh->VertexArray[pMesh->numVertices].x,
								&pMesh->VertexArray[pMesh->numVertices].y,
								&pMesh->VertexArray[pMesh->numVertices].z);
			pMesh->numVertices++;
		}

		/*
		 *	If "vn", read in normal
		 */
		if ( strcmp(buffer,"vn")==0 ) {
			fscanf(fp,"%f%f%f", &pMesh->NormalArray[pMesh->numNormals].x,
								&pMesh->NormalArray[pMesh->numNormals].y,
								&pMesh->NormalArray[pMesh->numNormals].z);
			pMesh->numNormals++;
		}

		/*
		 *	If "vt", read in texturing coord
		 */
		if ( strcmp(buffer,"vt")==0 ) {
			fscanf(fp,"%f%f",	&pMesh->uvarray[pMesh->numuv].u,
								&pMesh->uvarray[pMesh->numuv].v);
			pMesh->numuv++;
		}

		/*
		 *	If "f" read in the face indices. Note that the 
		 *	indices of the obj file start at 1, the indices
		 *	for the arrays start at zero, so remove 1 from all 
		 *	indices in the face.
		 */
		if ( strcmp(buffer,"f")==0) {
			fscanf(fp, "%d/%d/%d%d/%d/%d%d/%d/%d", 
				&pMesh->Faces[pMesh->numFaces].v[0],
				&pMesh->Faces[pMesh->numFaces].t[0],
				&pMesh->Faces[pMesh->numFaces].n[0],
				&pMesh->Faces[pMesh->numFaces].v[1],
				&pMesh->Faces[pMesh->numFaces].t[1],
				&pMesh->Faces[pMesh->numFaces].n[1],
				&pMesh->Faces[pMesh->numFaces].v[2],
				&pMesh->Faces[pMesh->numFaces].t[2],
				&pMesh->Faces[pMesh->numFaces].n[2]);

				--pMesh->Faces[pMesh->numFaces].v[0];
				--pMesh->Faces[pMesh->numFaces].t[0];
				--pMesh->Faces[pMesh->numFaces].n[0];
				--pMesh->Faces[pMesh->numFaces].v[1];
				--pMesh->Faces[pMesh->numFaces].t[1];
				--pMesh->Faces[pMesh->numFaces].n[1];
				--pMesh->Faces[pMesh->numFaces].v[2];
				--pMesh->Faces[pMesh->numFaces].t[2];
				--pMesh->Faces[pMesh->numFaces].n[2];

			pMesh->numFaces++;
		}  
	}
	fclose(fp);

	return pMesh->id;	
}

void savemod (char* filename, int id)
{
	mesh* p = getMesh(id);
	int i;
	FILE* fp;
	fp=fopen(filename, "w");

	for(i=0;i<p->numVertices; i++)
	{
		fprintf(fp, "v %f %f %f\n", p->VertexArray[i].x,p->VertexArray[i].y,p->VertexArray[i].z);
	}
	for(i=0;i<p->numuv; i++)
	{
		fprintf(fp, "vt %f %f\n", p->uvarray[i].u,p->uvarray[i].v);
	}
	for(i=0;i<p->numNormals; i++)
	{
		fprintf(fp, "vn %f %f %f\n", p->NormalArray[i].x,p->NormalArray[i].y,p->NormalArray[i].z);
	}
	for(i=0;i<p->numFaces; ++i)
	fprintf(fp, "f %d/%d/%d %d/%d/%d %d/%d/%d\n", 
				p->Faces[i].v[0]+1,
				p->Faces[i].t[0]+1,
				p->Faces[i].n[0]+1,
				p->Faces[i].v[1]+1,
				p->Faces[i].t[1]+1,
				p->Faces[i].n[1]+1,
				p->Faces[i].v[2]+1,
				p->Faces[i].t[2]+1,
				p->Faces[i].n[2]+1);


	fclose(fp);
}



void saveC (char* filename, int id)
{
	int i,j;
	mesh* p = getMesh(id);

	if(p!=NULL)
	{
		FILE* fp=fopen(filename, "w");

		fprintf (fp, "float VertData[] =\n{");

		for(i=0; i<p->numFaces; i++)
		{
			for(j=0; j<3; j++){

				fprintf (fp, "%f,%f,",p->uvarray[ p->Faces[i].t[j] ].u, p->uvarray[ p->Faces[i].t[j] ].v)   ;
				fprintf (fp, "%f,%f,%f,", p->NormalArray[ p->Faces[i].n[j] ].x, p->NormalArray[ p->Faces[i].n[j] ].y, p->NormalArray[ p->Faces[i].n[j] ].z);
				fprintf (fp, "%f,%f,%f,", p->VertexArray[ p->Faces[i].v[j] ].x, p->VertexArray[ p->Faces[i].v[j] ].y, p->VertexArray[ p->Faces[i].v[j] ].z);
			}
		}
		fprintf (fp, "0};");
		fclose (fp);
	}

}